//
// Light.control.44.c
//
// multilights control
//    
//
// cedric Doutriaux
// 2014
// cc-by-sa
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h> 

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define cycle_delay() _delay_us(2) // cycle delay
#define current 150 // PWM current
#define off 255 // PWM off

// output for lights
#define LeftLight_pin (1 << PA0)
#define Light_pin (1 << PA1)
#define RightLight_pin (1 << PA2)
#define Lights_port PORTA
#define Lights_direction DDRA

//input for buttons
#define LeftBton_pin (1 << PA4)
#define RightBton_pin (1 << PA6)
#define LightBton_pin (1 << PA5)
#define button_port PORTA
#define button_pins PINA

uint16_t blink_mode=0;//blinker mode /0:off /1:left /2:right /3:warning
uint16_t light_mode=0;//lightt mode /0:off /1:low /2:high
uint16_t btonLightState=0;

//unsigned char  blinkers;//list of blinker led actives


uint16_t blink_delay=600;//cs

void long_delay_ms(uint16_t ms) {
    for(ms /= 10; ms>0; ms--) _delay_ms(10);
}


void poormanPWM(){//for lights
int i;
     for (i=0; i<10; i++) {
           if(light_mode!=0){
               set(Lights_port,Light_pin); // Turn light on if necessary
           }else{
               clear(Lights_port,Light_pin);//else shut it
               }
           long_delay_ms(blink_delay/20);
           if(light_mode==1) clear(Lights_port,Light_pin); // Turn light off half time in low mode
           long_delay_ms(blink_delay/20);
           
        }
    
    }



ISR(PCINT0_vect)        // interrupt service routine
{              // called when PCINT0 changes state


  
  /////////////BLINKERS
  if((!(button_pins & LeftBton_pin)) && blink_mode==0 ){
      //blinkers= LeftLight_pin ;
      blink_mode=1;
  }
  
   if((!(button_pins & RightBton_pin)) && blink_mode==0  ){
      //blinkers= RightLight_pin ;
      blink_mode=2;
  }
    
 if((button_pins & RightBton_pin) && blink_mode==2){
     blink_mode=0;
     }
       
    if((button_pins & LeftBton_pin) && blink_mode==1){
     blink_mode=0;
     }
    
    
    /////////////////LIGHT
    if(!(button_pins & LightBton_pin) && btonLightState==0){
        //LightBton_pin rise up
        //
        btonLightState=1;
    }
    if((button_pins & LightBton_pin) && btonLightState==1){

        //LightBton_pin rise down
        btonLightState=0;
        //change light mode
        if(light_mode<2) {
            light_mode+=1;
        }else{
            light_mode=0;
        }       
    }
  
  
   return;
} 


int main(void) {
   //
   // main
   //
   
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize Light pins
   //
   clear(Lights_port, LeftLight_pin);
   clear(Lights_port, RightLight_pin);
   clear(Lights_port, Light_pin);
   output(Lights_direction, Light_pin);
   output(Lights_direction, LeftLight_pin);
   output(Lights_direction, RightLight_pin);
   
   // initialise button pins pullup
   //
   set(button_port,LeftBton_pin); 
   set(button_port,RightBton_pin) ;
   set(button_port,LightBton_pin); 
    //initialise inputs
   set(Lights_direction,LeftBton_pin); 
   set(Lights_direction,RightBton_pin); 
   set(Lights_direction,LightBton_pin) ;
  
   
    GIMSK |= (1<<PCIE0);   // enable PCIE0 interrupt 
    PCMSK0 = (1<<PCINT4)|(1<<PCINT5)|(1<<PCINT6);   // pin change mask: listen to portA 4, 5 and 6
    sei();				//Enable Global Interrupt
   //
   // main loop
   //
   while (1) {
      //
    
      
      
      
      
      ///////////////////////////////////////////BLINK HIGH
      if(blink_mode!=0){
          //turn blinkers on if necessary           
          if(blink_mode==1)set(Lights_port,LeftLight_pin); 
          if(blink_mode==2)set(Lights_port,RightLight_pin); 
      }else{
          clear(Lights_port,LeftLight_pin); 
          clear(Lights_port,RightLight_pin); 
      }
        
      poormanPWM();
        
        /////////////////////////////////////////BLINK LOW
           if(blink_mode!=0){//turn blinkers off
          if(blink_mode==1)clear(Lights_port,LeftLight_pin); 
          if(blink_mode==2)clear(Lights_port,RightLight_pin); 
         }
          
         poormanPWM(); 
          
          
          
          
          
          
            
            
       
   }
}
